#!/usr/bin/env python
#
# Author:  Yotam Medini  yotam.medini@gmail.com -- Created: 2006/November/27
#
# Solve system of 2 linear equations with 2 unknowns


# standard
import sys
import string

# local
import fraction


def usage(a0):
    sys.stderr.write("""
Usage:
%s <file name>

Where input file has two lines, with numbers
  a b c
  d e f
representing the equations
  ax + by = c
  dx + ey = f
or lines starting with '#' for comments.
The numbers are either decimal integers ar integer fractions like  -3/2
See lineq2.inp for input example.
""" % (a0))
    sys.exit(0)


def get_data_line(f):
    "Get non-comment line from f or empty string for end of input"
    line = '#'
    while line != '' and line[0] == '#':
        line = f.readline()
    return line


def get3fractions_from_line(line):
    "return 3 fractions from line, None for failure"
    ret = None
    ss = string.split(line)
    if len(ss) == 3:
        f3 = 3*[None]
        ret = f3
        for i in range(0, 3):
            f3[i] = fraction.Fraction()
            ok = f3[i].strset(ss[i])
            if not ok:
                ret = None
    if ret == None:
        sys.stderr.write("Failed to get 3 fractions from %s" % line)
    return ret


### Program begin
rc = 0
if len(sys.argv) != 2:
    usage(sys.argv[0])
fn = sys.argv[1]
f = open(fn, 'r')
abc_line = get_data_line(f)
def_line = get_data_line(f)
f.close()
if abc_line == '' or def_line == '':
    sys.stderr.write("Missing data lines\n")
    usage(sys.argv[0])
f3_abc = get3fractions_from_line(abc_line)
f3_def = get3fractions_from_line(def_line)
if f3_abc == None  or  f3_def == None:
    usage(sys.argv[0])

# Now we have the numbers!
a = f3_abc[0];  b = f3_abc[1];  c = f3_abc[2];
d = f3_def[0];  e = f3_def[1];  f = f3_def[2];

det = a*e - d*b
if det == fraction.zero: # Singular case
    if a*f == e*c:
        sys.stdout.write("Infinitely many solutions\n")
    else:
        sys.stdout.write("No solution\n")
else:
    x = (c*e - b*f) / det
    y = (a*f - c*d) / det

    # Now just test:
    ok1 = (a*x + b*y == c)
    ok2 = (d*x + e*y == f)
    if not (ok1 and ok2):
        sys.stderr.write("Program bug !!! Call programmer and complain!\n")
        sys.exit(1)

    sys.stdout.write("Solution:  x = %s   y = %s\n" % (x, y))
sys.exit(rc)
